; ------------- READS51 generated header --------------
; module  : syscalls.asm
;
; Courtesy Rigel Corporation
;
; www.rigelcorp.com
;
; Copyright (C) 1988-2002 Rigel Corporation
;
;
; !!! Do not assemble this file as is. !!!
;
; The routines given here are samples. You may 
; incorporate them into MinMon.asm to increase 
; the functionality of your monitor program.
;
; Portions of this code are explained in the textbook
; Programming and Interfacing the 8051 in C and Assembly
; by S. Yeralan and H. Emery
; (C) 2000, Rigel Press, www.rigelcorp.com
; -----------------------------------------------------


; ==========================================================
; subroutine ascbin
; this routine takes the ascii character passed to it in the
; acc and converts it to a 4 bit binary number which is returned 
; in the acc.
; ==========================================================
ascbin:  add   a,  #0d0h        ; if chr < 30 then error
         jnc   notnum
         clr   c                 ; check if chr is 0-9
         add   a,  #0f6h        ; adjust it
         jc    hextry            ; jmp if chr not 0-9
         add   a,  #0ah         ; if it is then adjust it
         ret

hextry:  clr   acc.5             ; convert to upper
         clr   c                 ; check if chr is a-f
         add   a,  #0f9h        ; adjust it
         jnc   notnum            ; if not a-f then error
         clr   c                 ; see if char is 46 or less.
         add   a,  #0fah        ; adjust acc
         jc    notnum            ; if carry then not hex
         anl   a,  #0fh         ; clear unused bits
         ret

notnum:  setb  errorf            ; if not a valid digit
         ret



; ====================================================
;  subroutine autoexec start up program
;  scans low rom memory to see if an embedded program
;  is to run at power up.
; ----------------------------------------------------
autoexec:
       mov   psw,  #0      ; select register bank 0
       mov   dptr, #ax_tab
ax_0:
       movx  a,   @dptr    ; read low  address
       mov   r0,  a
       inc   dptr
       movx  a,   @dptr    ; read high address
       mov   r1,  a
       anl   a,   r0
       cpl   a
       jz    ax_dn
       mov   a,   r1
       orl   a,   r0
       jz    ax_1
; ---- start up program specified -------------------
; ---- allow user to abort --------------------------

       push  0            ; lsb to print next
       push  1            ; msb to print next
       lcall crlf
       lcall print
       db   "about to execute program at ",0
       pop   acc      ; recall msb
       lcall prthex
       pop   acc      ; recall lsb
       lcall prthex
       lcall crlf
       lcall print
       db   "hit any key to abort", 0
       clr   ri
       lcall sdelay       ; 2-second grace period
       jb    ri, ax_dn
       lcall sdelay
       jb    ri, ax_dn    ; if key hit the abort...

       lcall crlf         ; ...else start up prog
       pop   acc          ; flush return address ...
       pop   acc          ; ... off from stack
       push  0            ; lsb of start up program
       push  1            ; msb of start up program
       ret                ; effectively jumps to prog

ax_dn: ret
ax_1:  inc   dptr
       ljmp  ax_0


; ====================================================
;  subroutine beep
;  input    : none
;  output   : ^g (bell = 7) sent to serial port
;  destroys : a
; ----------------------------------------------------
beep:   push  acc
        mov a, #7h
        lcall sndchr
        pop   acc
        ret


; ==========================================================
; subroutine binasc
; binasc takes the contents of the accumulator and converts it
; into two ascii hex numbers.  the result is returned in the 
; accumulator and r2.
; ==========================================================
;
binasc:  mov   r2, a             ; save in r2
         anl   a,  #0fh         ; convert least sig digit.
         add   a,  #0f6h        ; adjust it
         jnc   noadj1            ; if a-f then readjust
         add   a,  #07h
noadj1:  add   a,  #3ah         ; make ascii

         xch   a,  r2            ; put result in reg 2
         swap  a                 ; convert most sig digit
         anl   a,  #0fh         ; look at least sig half of acc
         add   a,  #0f6h        ; adjust it
         jnc   noadj2            ; if a-f then re-adjust
         add   a,  #07h
noadj2:  add   a,  #3ah         ; make ascii
         ret


; ==========================================================
; subroutine chkbrk
; this routine checks for the break key. if it is found control
; is passed back to the main monitor loop.
; ==========================================================
;
chkbrk:  jnb   ri, nobrk               ; if no chr then return
         mov   a,  sbuf                ; get chr from serial port
         clr   ri                      ; reset rx status bit
break:   cjne  a,  #03h,nobrk         ; if cnt c then
         lcall print                   ; print 'break'
         db 0dh, 0ah," <break> ", 000h
         ljmp  return                  ; return to monitor
nobrk:   ret                           ; else normal return


; ==========================================================
; subroutine crlf
; crlf sends a carriage return line feed out the serial port
; ==========================================================
;
crlf:    mov   a,  #0ah         ; print lf
         lcall sndchr
cret:    mov   a,  #0dh         ; print cr
         lcall sndchr
         ret


; ====================================================
;  subroutine delay - millisecond delay
;   accumulator holds microseconds to delay
;   - 2 microseconds are reserved for the call
;   to this routine.
;  input    : milliseconds to delay in accumulator
;  output   : none
;  destroys : nothing - uses a
; ----------------------------------------------------
; 100h-a6h=5ah=(90)decimal
; 90 * 11 = 990
; plus 10 gives 1 millisecond microsecond
;
;                        microseconds (cycles)
;                       -----------------------
delay:
        dec   a         ;    1

d_olp:
      push acc          ;    2            \
      mov   a, #0a6h    ;    1            |
                        ;                 |
d_ilp:  inc   a         ;    1  \         |
        nop             ;    1  |         |
        nop             ;    1  |         |
        nop             ;    1  |         |
        nop             ;    1  |         |
        nop             ;    1  |- 11     | (acc-1)
        nop             ;    1  | cycles  |- msec
        nop             ;    1  |         |
        nop             ;    1  |         |
        jnz   d_ilp     ;    2  /         |
                        ;                 |
        nop             ;    1            |
        nop             ;    1            |
        nop             ;    1            |
        pop   acc       ;    2            |
                        ;                 |
        djnz  acc,d_olp ;  2            /

; need to wait 998 microseconds more

        mov   a, #0a6h  ;    1

d_lp2:  inc   a         ;    1  \
        nop             ;    1  |
        nop             ;    1  |
        nop             ;    1  |
        nop             ;    1  |
        nop             ;    1  |- 11
        nop             ;    1  | cycles
        nop             ;    1  |
        nop             ;    1  |
        jnz   d_lp2     ;    2  /
        nop             ;    1
        nop             ;    1
        nop             ;    1
        nop             ;    1
        nop             ;    1

        ret             ;    2


; ====================================================
;  subroutine display - display string
;  input    : nibble in accumulator
;  output   : 7-segment pattern in accumulator
;             (acc.0 is segment a, acc.6 is segment g)
;  destroys : a
; ----------------------------------------------------
display:
        inc  a
        movc a, @a+pc
        ret
        db  0c0h  ; 0
        db  0f9h  ; 1
        db  0a4h  ; 2
        db  0b0h  ; 3
        db  99h  ; 4
        db  92h  ; 5
        db  82h  ; 6
        db  0f8h  ; 7
        db  80h  ; 8
        db  90h  ; 9
        db  88h  ; a
        db  83h  ; b
        db  0c6h  ; c
        db  0a1h  ; d
        db  86h  ; e
        db  8eh  ; f


; ==========================================================
; subroutine getbyt
; this routine reads in an 2 digit ascii hex number from the
; serial port. the result is returned in the acc.
; ==========================================================
;
getbyt:  lcall getchr            ; get msb ascii chr
         lcall ascbin            ; conv it to binary
         swap  a                 ; move to most sig half of acc
         mov   b,  a             ; save in b
         lcall getchr            ; get lsb ascii chr
         lcall ascbin            ; conv it to binary
         orl   a,  b             ; combine two halves
         ret


; ==========================================================
; subroutine getchr
; this routine reads in a chr from the serial port and saves it
; in the accumulator.
; ==========================================================
;
getchr:  jnb  ri, getchr         ; wait till character received
         mov  a,  sbuf           ; get character
         anl  a,  #7fh          ; mask off 8th bit
         clr  ri                 ; clear serial status bit
         ret


; ==========================================================
; subroutine getchrx
; this routine reads in a chr from the serial port and 
; saves it in the acc.  it differs from getchr by not
; clearing the most significant bit.
; ==========================================================
;
getchrx: jnb  ri, getchrx  ; wait till character received
         mov  a,  sbuf     ; get character
         clr  ri           ; clear serial status bit
         ret


; ====================================================
;  subroutine inkey
;  input    : peeks at serial port - no wait
;  output   : if chr present
;                 then accumulator returns chr
;                 else accumulator is 00 (null)
;  destroys : a
; ----------------------------------------------------
inkey:  mov a,  #00h
        jnb ri, doneik
        mov a,  sbuf
        anl a, #7fh
        clr ri
doneik: ret


; ==========================================================
; subroutine init
; this routine intializes the hardware on the 8051
; ==========================================================
;
init:
           mov psw,  #0h            ; select rb0
           mov dptr, #int0           ; point to interrupt vector table
           mov p2,   #ofsthi         ; offset high byte
           mov r0,   #80h           ; offset into stored rom vector table

transfer:  movx a, @r0               ; transfer...
           movx @dptr, a             ; ...rom interrupt vector...
           inc  r0                   ; ...table to...
           inc dptr                  ; ...ram (af = 80 + 2f)
           cjne r0, #0afh, transfer

           clr  dashf                ; initialize software flag
           clr  equalf               ; initialize software flag
           clr  breakf               ; initialize software flag
           clr  hostf                ; initialize software flag
; - - - - - - set up serial port - - - - - -
; with a 11.059 Mhz crystal, use timer 1 as the Baud rate generator
; for 9600 Baud

	   mov   tmod, #20h         ; set timer 1 for auto reload - mode 2
	   mov   tcon, #41h         ; run timer 1 and set edge trig ints
	   mov   th1,  #0fdh        ; set timer 1 for 9600 baud with xtal=12mhz
	   mov   scon, #50h         ; set serial control reg for 8 bit data
				    ; and mode 1
	   ret

; ====================================================
;  subroutine mdelay - millisecond delay
;   delays for 998 microseconds - 2 microseconds are
;   reserved for the call to this routine.
;  input    : none
;  output   : none
;  destroys : nothing - uses a
; ----------------------------------------------------
; 100h-a6h=5ah=(90)decimal
; 90 * 11 = 990
; plus 8 gives 998 microseconds
;
;                        microseconds (cycles)
;                       -----------------------
mdelay: push acc        ;    2
        mov   a, #0a6h  ;    1

md_olp: inc   a         ;    1  \
        nop             ;    1  |
        nop             ;    1  |
        nop             ;    1  |
        nop             ;    1  |
        nop             ;    1  |- 11  cycles
        nop             ;    1  |
        nop             ;    1  |
        nop             ;    1  |
        jnz   md_olp    ;    2  /

        nop             ;    1
        pop   acc       ;    2
        ret             ;    2


; ====================================================
;  subroutine percent - converts [0-ff] to [0-99] bcd
;  input    : byte in accumulator
;  output   : [0-99] bcd in accumulator
;
;  destroys : a, r0, dptr, carry flag
; ----------------------------------------------------
percent:
        cjne a, #0ffh, anotff
        mov  a, #99h
        ret

anotff: inc  a
        movc a, @a+pc
        ret
; table of bcd corresponding to the byte
 db 00h, 00h, 01h, 01h, 02h, 02h, 02h, 03h  ;  0 -  7
 db 03h, 04h, 04h, 04h, 05h, 05h, 05h, 06h  ;  8 -  f
 db 06h, 07h, 07h, 07h, 08h, 08h, 09h, 09h  ; 10 - 17
 db 09h, 10h, 10h, 11h, 11h, 11h, 12h, 12h  ; 18 - 1f
 db 12h, 13h, 13h, 14h, 14h, 14h, 15h, 15h  ; 20 - 27
 db 16h, 16h, 16h, 17h, 17h, 18h, 18h, 18h  ; 28 - 2f
 db 19h, 19h, 20h, 20h, 20h, 21h, 21h, 21h  ; 30 - 37
 db 22h, 22h, 23h, 23h, 23h, 24h, 24h, 25h  ; 38 - 3f

 db 25h, 25h, 26h, 26h, 27h, 27h, 27h, 28h  ; 40 - 47
 db 28h, 29h, 29h, 29h, 30h, 30h, 30h, 31h  ; 48 - 4f
 db 31h, 32h, 32h, 32h, 33h, 33h, 34h, 34h  ; 50 - 57
 db 34h, 35h, 35h, 36h, 36h, 36h, 37h, 37h  ; 58 - 5f
 db 37h, 38h, 38h, 39h, 39h, 39h, 40h, 40h  ; 60 - 67
 db 41h, 41h, 41h, 42h, 42h, 43h, 43h, 43h  ; 68 - 6f
 db 44h, 44h, 45h, 45h, 45h, 46h, 46h, 46h  ; 70 - 77
 db 47h, 47h, 48h, 48h, 48h, 49h, 49h, 50h  ; 78 - 7f

 db 50h, 50h, 51h, 51h, 52h, 52h, 52h, 53h  ; 80 - 87
 db 53h, 54h, 54h, 54h, 55h, 55h, 55h, 56h  ; 88 - 8f
 db 56h, 57h, 57h, 57h, 58h, 58h, 59h, 59h  ; 90 - 97
 db 59h, 60h, 60h, 61h, 61h, 61h, 62h, 62h  ; 98 - 9f
 db 62h, 63h, 63h, 64h, 64h, 64h, 65h, 65h  ; a0 - a7
 db 66h, 66h, 66h, 67h, 67h, 68h, 68h, 68h  ; a8 - af
 db 69h, 69h, 70h, 70h, 70h, 71h, 71h, 71h  ; b0 - b7
 db 72h, 72h, 73h, 73h, 73h, 74h, 74h, 75h  ; b8 - bf

 db 75h, 75h, 76h, 76h, 77h, 77h, 77h, 78h  ; c0 - c7
 db 78h, 79h, 79h, 79h, 80h, 80h, 80h, 81h  ; c8 - cf
 db 81h, 82h, 82h, 82h, 83h, 83h, 84h, 84h  ; d0 - d7
 db 84h, 85h, 85h, 86h, 86h, 86h, 87h, 87h  ; d8 - df
 db 87h, 88h, 88h, 89h, 89h, 89h, 90h, 90h  ; e0 - e7
 db 91h, 91h, 91h, 92h, 92h, 93h, 93h, 93h  ; e8 - ef
 db 94h, 94h, 95h, 95h, 95h, 96h, 96h, 96h  ; f0 - f7
 db 97h, 97h, 98h, 98h, 98h, 99h, 99h, 99h  ; f8 - ff


; ==========================================================
; subroutine print
; print takes the string immediately following the call and 
; sends it out the serial port.  the string must be terminated
; with a null. this routine will ret to the instruction
; immediately following the string.
; ==========================================================
;
print:   pop   dph              ; put return address in dptr
         pop   dpl
         lcall prtstr           ; print string and update dptr
         mov   a,  #1h          ; point to instruction after string
         jmp   @a+dptr          ; return


; ==========================================================
; subroutine prtstr
; this routine takes the string pointed to by the data pointer
; and sends it out the serial port. the string must be
; terminated with a null.
; ==========================================================
;
prtstr:  clr  a                  ; set offset = 0
         movc a,  @a+dptr        ; get chr from code memory
         cjne a,  #0h, mchrok    ; if chr = ff then return
         ret
mchrok:  lcall sndchr            ; send character
         inc   dptr              ; point at next character
         ljmp  prtstr            ; loop till end of string


; ==========================================================
; subroutine prthex
; this routine takes the contents of the acc and prints it out 
; as a 2 digit ascii hex number.
; ==========================================================
;
prthex:  lcall binasc            ; convert acc to ascii
         lcall sndchr            ; print first ascii hex digit
         mov   a,  r2            ; get second ascii hex digit
         lcall sndchr            ; print it
         ret


; ==========================================================
; subroutine prsphx
; this routine first prints a space then takes the contents of
; the acc and prints it out as a 2 digit ascii hex number.
; ==========================================================
;
prsphx:  mov   r2, a             ; save acc in r2
         mov   a,  #20h         ; print space
         lcall sndchr
         mov   a,  r2            ; recall acc
         lcall prthex            ; print it
         ret


; ====================================================
;  subroutine sdelay - second delay
;   delays for 999998 microseconds - 2 microseconds
;   are reserved for the call to this routine.
;  input    : none
;  output   : none
;  destroys : nothing - uses a
; ----------------------------------------------------
; 100h-91h=6fh=(111)decimal
; 9008 * 111 = 999888
; plus 102 from second loop
; plus 8 gives 999998 microseconds
;
;                        microseconds (cycles)
;                       -----------------------
sdelay: push acc             ;   2
        mov  a, #91h        ;   1

sd_olp: inc   a      ; \
        lcall mdelay ; |
        lcall mdelay ; |
        lcall mdelay ; |
        lcall mdelay ; |
        lcall mdelay ; |
        lcall mdelay ; |
        lcall mdelay ; |- loop takes 9008 microseconds
        lcall mdelay ; |
        lcall mdelay ; |
        nop          ; |
        nop          ; |
        nop          ; |
        nop          ; |
        nop          ; |
        jnz   sd_olp ; /

        mov   a, #33h       ;   1
sd_ilp: djnz  acc, sd_ilp    ; -loop takes 2*33h=66h=(102)dec

        pop   acc            ;   2
        ret                  ;   2


; ==========================================================
;  subroutine setintvec - set interrupt vector
;  input    : a contains interrupt source [0..11]
;             dptr contains isr address
;  output   : none
;  destroys : a, dptr
; ----------------------------------------------------------
setintvec:
        push dpl            ; push isr address
        push dph
        anl  a, #0fh        ; just to be sure
        rl   a              ; multiply by 4
        rl   a
        mov  dph, #0ffh     ; ram vector table
        mov  dpl, a         ; dptr points to vector table
        mov  a, #2          ; ljmp instruction
        movx @dptr, a 
        inc  dptr
        pop  acc            ; pop isr address high byte
        movx @dptr, a
        inc  dptr
        pop  acc            ; pop isr address low byte
        movx @dptr, a       ; new int vector placed
        ret


; ==========================================================
; subroutine sndchr
; this routine takes the chr in the acc and sends it out the
; serial port.
; ==========================================================
;
sndchr:  clr  scon.1             ; clear the tx  buffer full flag.
         mov  sbuf,a             ; put chr in sbuf
txloop:  jnb  scon.1, txloop     ; wait till chr is sent
         ret
; ====================================================

; ====================================================
; The following are not part of the system ROM
; To use, include them in your source code
; ====================================================
;  subroutine kbdclrw -
;              clear keyboard and wait for keypressed
;  input    : none
;  output   : none
;  destroys : nothing - uses a
; ----------------------------------------------------
kbdclrw:
        push  acc
kcw_0:
        jnb   ri, kcw_1
        clr   ri
        sjmp  kcw_0
kcw_1:
        lcall getchr
        pop   acc
        ret

; ====================================================
;  subroutine kbdwait - waits for keypressed
;  input    : none
;  output   : none
;  destroys : nothing - uses a
; ----------------------------------------------------
kbdwait:
        push  acc
        lcall getchr
        pop   acc
        ret

; ==========================================================
; subroutine getword
; this subrouinte reads in a string in ascii from the serial
; port.  the line must be terminated with a <cr>.  the line is
; stored in the line buffer. the maximum line length is 16
; bytes including the <cr>.  the word is returned in the first
; parameter buffer at location pbuffr.
; ==========================================================
;
getword:
       push  psw
       mov   psw, #0
       push  3
       push  4
       push  5
       push  6

       clr   errorf
       clr   ri                     ; flush serial port
       mov   r0,  #lbuffr           ; init line buffer ram to 0's
gwinit:
       mov   @r0, #0ffh
       inc   r0
       cjne  r0,  #lbuffr+11h, gwinit

       mov   r4,  #0h              ; init line buffer count (pointer)
gw0:   mov   a,   r4                ; if line length > 15 then error
       jnb   acc.4, gwok0
       ljmp  gwerr_ret

gwok0: lcall getchr                 ; read in chr
       mov   r1, a                  ; save in r1
       cjne  r1, #7fh, gwnodel     ; if del then del
       sjmp  gwdel
gwnodel:
       cjne  r1, #08h, gwnorub     ; if back space then del
gwdel: mov   a,  r4                 ; do not back up over prompt
       jz    gw0
       mov   a,  #08h              ; backspace
       lcall sndchr
       mov   a,  #20h              ; send a space
       lcall sndchr
       mov   a,  #08h              ; backspace
       lcall sndchr
       dec   r4                     ; dec line buffer pointer.
       sjmp  gw0
gwnorub:
       mov   a, r1                  ; recall char
       clr   c
       add   a, #0c0h
       jnc   gw_num                 ; if alphabetic then
       mov   a, r1                  ; then make upper case
       clr   acc.5
       mov   r1, a
gw_num:
       mov   a,  r4                 ; save chr in line buffer.
       add   a,  #lbuffr            ; compute address
       mov   r0, a
       mov   a,  r1                 ; get chr
       mov   @r0,a
       lcall sndchr                 ; echo character

       inc   r4                     ; point to next location in buffer
       cjne  r1, #0dh, gw0         ; if not cr then return
       mov   a,  #0ah              ; if chr=cr then line feed and do normal ret
       lcall sndchr
       mov   a,  r4                 ; if cr only then error
       cjne  a,  #1h, gwok
       ljmp  gwerr_ret
gwok:
     mov   r0, #lbuffr-1       ; point before first chr in line buffer

; - - - - - -  read in parameter  - - - - - - - - - - -
gw1:
       inc   r0                 ; point to next location in line buffer
       mov   a,  @r0            ; get chr from line buffer
       cjne  a,  #0dh, gw2     ; if cr then end
       ljmp  gwerr_ret          ; return on error

gw2:   cjne  a,  #20h, gw3     ; if chr=space then loop
       sjmp  gw1
gw3:
       mov   r1, #pbuffr        ; parameter 0
       lcall getparx            ; get parameter from line buffer
       jnb   errorf, gwret

gwerr_ret:
       setb  errorf
       lcall subbadpar

gwret: pop   6
       pop   5
       pop   4
       pop   3
       pop   psw
       ret

